@@ -11,7 +11,13 @@ def rand_y():
   return random_position(20, 500)
 
 class Block:
-  
+ 
+  """
+  A block is a StreamTools module,
+  It can be connected with other blocks
+  to form a StreamTools Connection
+  """
+ 
   def __init__(self, 
       id = None, 
       type = None, 
@@ -25,17 +31,19 @@ class Block:
     # get id 
     self.id = id
 
-    # initialize client.
+    # initialize client + library.
     self.url = kw.get('url', settings.STREAMTOOLS_URL)
     self._st = Api(self.url)
+    self._lib = self._st.library()
 
     # pull in existing block
-    if self.exists():
+    if self.is_attached():
       kw['raw'] = self._st.get_block(self.id)
 
     # check for raw switch
     if not 'raw' in kw:
       
+      # assign in kwargs
       self.id = id
       self.type = type
       self.rule = rule
@@ -48,7 +56,7 @@ class Block:
       self.type = kw['raw'].get('Type', type)
       self.rule = kw['raw'].get('Rule', rule)
       
-      # position
+      # parse position
       pos = kw['raw'].get('Position')
       if pos:
         self.x_pos = pos.get('X', rand_x())
@@ -64,10 +72,20 @@ class Block:
 
   @property 
   def position(self):
+   
+    """
+    The raw, api-friendly representation of this Block's position.
+    """
+   
     return {'X': self.x_pos, 'Y': self.y_pos}
 
   @property 
   def raw(self):
+
+    """
+    The raw, api-frienfly representation of this Block.
+    """
+
     return {
       'Id': self.id,
       'Type': self.type,
@@ -77,6 +95,11 @@ class Block:
 
   @property 
   def in_blocks(self):
+    
+    """
+    The block ids this Block recieves messages from.
+    """
+
     block_ids = []
     for c in self._st.list_connections():
       if c['ToId'] == self.id:
@@ -85,6 +108,11 @@ class Block:
 
   @property 
   def out_blocks(self):
+    
+    """
+    The block ids this Block sends messages to.
+    """
+    
     block_ids = []
     for c in self._st.list_connections():
       if c['FromId'] == self.id:
@@ -93,31 +121,43 @@ class Block:
 
   @property 
   def in_routes(self):
-    resp = self._st.library()
-    resp = resp.get(self.id, {})
+
+    """
+    This Block's available InRoutes
+    """
+
+    resp = self._lib.get(self.id, {})
     return resp.get('InRoutes', [])
 
   @property 
   def out_routes(self):
+    
+    """
+    This Block's available OutRoutes
+    """
+
     resp = self._st.library()
     resp = resp.get(self.id, {})
     return resp.get('OutRoutes', [])
 
   @property 
   def query_routes(self):
+    
+    """
+    This Block's available QueryRoutes
+    """
+
     resp = self._st.library()
     resp = resp.get(self.id, {})
     return resp.get('QueryRoutes', [])
 
-  def stop(self):
-    try:
-      self._st.delete_block(self.id)
-      return True
-    except ValueError:
-      return False
-
-  def start(self):
-    # create connection, reset id
+  def attach(self):
+   
+    """
+    Attach this Block to streamtools and update it's id.
+    """
+   
+    # create block, reset id
     self.id = self._st.create_block(
                 self.id, 
                 type = self.type, 
@@ -126,39 +166,100 @@ class Block:
                 y_pos = self.y_pos
               )  
 
-  def exists(self):
+  def detach(self):
+    """
+    Detach this Block from streamtools.
+    """
+    
+    try:
+      self._st.delete_block(self.id)
+      return True
+    
+    except ValueError:
+      return False
+
+  def is_attached(self):
+    """
+    Check if this Block is attached to streamtools.
+    """
+    
     try:
       self._st.get_block(self.id)
       return True 
+    
     except ValueError:
       return False
 
   def refresh(self, overwrite=True):
-    if self.exists():
-      self.stop()
-    self.start()
+    """
+    Check if this Block is attached to streamtools,
+    if so, detach it first. Always attach it.
+    """
+    
+    if self.is_attached():
+    
+      self.detach()
 
-  def send_to(self, route='rule', msg={}):
-    return self._st.to_block_route(self.id, route=route, msg=msg)
+    self.attach()
 
-  def recieve_from(self, route='out'):
+  def send_to(self, **kw):
+    """
+    Send a msg to this Block's specified route.
+    """
+    
+    # parse kwargs
+    assert('msg' in kw)
+    kw.setdefault('route', 'rule')
+
+    return self._st.to_block_route(
+            self.id, 
+            route=route, 
+            msg=msg
+          )  
+
+  def recieve_from(self, **kw):
+    
+    """
+    Recieve a msg from this Block's specified route.
+    """
+
+    kw.setdefault('route', 'rule')
     return self._st.from_block_route(self.id, route=route)
 
   def listen(self):
+    
+    """
+    Listen to the output channel of this Block.
+    """
+    
     return self._st.stream(self.id)
 
   def __add__(self, obj):
+   
+    """
+    Methods for creating Connections 
+    between this Block and other Blocks.
+    """
 
     if isinstance(obj, Block):
       return Connection(from_id=self.id, to_id=obj.id)
 
   def __repr__(self):
+    
+    """
+    Block display
+    """
+    
     return "< Block.{} = {}, {} >"\
       .format(self.id, self.type, self.rule)
 
 
 class Connection:
   
+  """
+  A Connection is a linkage of two Blocks.
+  """
+  
   def __init__(self, 
       id = None, 
       from_id = None, 
@@ -166,6 +267,7 @@ class Connection:
       to_route = "in",
       **kw
     ):
+
     # get id 
     self.id = id
 
@@ -173,9 +275,9 @@ class Connection:
     self.url = kw.get('url', settings.STREAMTOOLS_URL)
     self._st = Api(self.url)
 
-    # pull in existing block
-    if self.exists():
-      kw['raw'] = self._st.get_block()
+    # pull in existing block if it exists.
+    if self.is_attached():
+      kw['raw'] = self._st.get_block(self.id)
 
     if not 'raw' in kw:
 
@@ -184,6 +286,7 @@ class Connection:
       self.to_route = to_route
 
     else:
+
       self.id = kw['raw'].get('Id', id)
       self.from_id = kw['raw'].get('FromId', from_id)
       self.to_id = kw['raw'].get('ToId', to_id)
@@ -195,9 +298,11 @@ class Connection:
 
   @property 
   def raw(self):
+    
     """
     A mapping back to the raw representation of a Connection.
     """
+    
     return {
       'Id': self.id,
       'FromId': self.from_id,
@@ -207,12 +312,17 @@ class Connection:
 
   @property 
   def blocks(self):
+    
+    """
+    Blocks associated with this Connection
+    """
+
     return [Block(self.from_id, _init=False), Block(self.to_id, _init=False)]
 
-  def stop(self):
+  def detach(self):
 
     """
-    Remove the connection from the Api.
+    Remove the Connection from the Api.
     """
 
     if self.id:
@@ -221,9 +331,10 @@ class Connection:
       except ValueError:
         pass
 
-  def start(self):
+  def attach(self):
+    
     """
-    Initialize the connection via the api
+    Initialize the Connection via the api
     """
 
     # create connection, reset id
@@ -234,25 +345,41 @@ class Connection:
                 to_route = self.to_route
               )
 
-  def refresh(self):
+  def is_attached(self):
+
     """
-    Update the api to current state of the object.
+    Check if this Connection is attached to streamtools,
     """
 
-    self.stop()
-    self.start()
-
-  def exists(self):
     try:
       self._st.get_connection(self.id)
       return True 
     except ValueError:
       return False
 
+  def refresh(self):
+   
+    """
+    Check if this Connection is attached to streamtools,
+    if so, detach it first. Always attach it.
+    """
+    
+    if self.is_attached():
+      
+      self.detach()
+
+    self.attach()
+
   def recieve_from(self, route="last"):
+
+    """
+    Recieve a msg from this Block's specified route.
+    """
+
     return self._st.from_connection_route(self.id, route=route)
     
   def listen(self, interval=1, max_buffer=60):
+    
     """
     Poll the last route for new results
     """
@@ -281,13 +408,12 @@ class Connection:
 
 
   def __add__(self, obj):
+    
     """
-    Logic for adding Blocks and Patterns.
+    Logic for adding Patterns + Connections.
     """
-    if isinstance(obj, Block):
-      return Connection(from_id=self.id, to_id=obj.id)
-
-    elif isinstance(obj, Pattern):
+    
+    if isinstance(obj, Pattern):
       obj.connections.append(self)
       self.blocks.extend(self.blocks)
       return obj
@@ -296,12 +422,22 @@ class Connection:
       return Pattern(connections = [obj, self])
 
   def __repr__(self):
+
+    """
+    Connection display
+    """
+
     return "< Connection.{} = Block.{} => Block.{}.{} >"\
       .format(self.id, self.from_id, self.to_id, self.to_route)
 
 
 class Pattern:
 
+  """
+  A Pattern is a collection of one or more Connections 
+  and the distinct Blocks associated with them.
+  """
+
   def __init__(self,
       connections = [],
       **kw
@@ -315,23 +451,18 @@ class Pattern:
     self._st = Api(self.url)
 
     # global in/out blocks.
-    self.inblock = None
-    self.inroute = kw.get('inroute', 'in')
-    self.outblock = None
-    self.outroute = kw.get('outroute', 'out')
+    self.in_block = kw.get('in_block', None)
+    self.in_route = kw.get('in_route', 'in')
+    self.out_block = kw.get('out_block', None)
+    self.out_route = kw.get('out_route', 'out')
       
-    if 'outblock' in kw:
-      self.outblock = kw['outblock']
-
-    elif 'inblock' in kw:
-      self.inblock = kw['inblock']
-
-    elif len(self.blocks):
-      self.inblock = self.blocks[0]
-      self.outblock = self.blocks[-1]
-
   @property 
   def raw(self):
+
+    """
+    A mapping back to the raw representation of a Pattern.
+    """
+    
     return {
       'Connections': [c.raw for c in self.connections],
       'Blocks': [b.raw for b in self.blocks]
@@ -339,46 +470,87 @@ class Pattern:
   
   @property 
   def connection_ids(self):
+
+    """
+    A list of Connection ids associated with this Pattern.
+    """
+    
     return [c.id for c in self.connections]
 
   @property 
   def block_ids(self):
+
+    """
+    A list of Block ids associated with this Pattern.
+    """
+
     return [b.id for b in self.blocks]
 
-  def stop(self):
+  def detach(self):
 
-    # only delete blocks + connection associated 
-    # with this pattern
+    """
+    Detach this Pattern from streamtools. 
+    Only delete Blocks + Connection associated 
+    with this pattern.
+    """
 
+    # connections
     for c in self.connections:
       try:
         self._st.delete_connection(c.id)
       except ValueError:
         pass
-      
-      for b in self.blocks:
-        try:
-          self._st.delete_block(b.id)
-        except ValueError:
-          pass 
+    
+    # blocks
+    for b in self.blocks:
+      try:
+        self._st.delete_block(b.id)
+      except ValueError:
+        pass 
+
+  def attach(self):
+
+    """
+    Attach this Pattern to streamtools. 
+    """
 
-  def start(self):
     self._st.set_pattern(self.raw)
 
   def refresh(self, overwrite=True):
-    self.stop()
-    self.start()
+
+    """
+    Refresh this Pattern. 
+    """
+
+    self.detach()
+    self.attach()
 
   def send_to(self, msg):
-    inblock = [b for b in self.blocks if b.id == self.inblock][0]
-    return self._st.to_block_route(inblock.id, route=self.inroute, msg=msg)
+    
+    """
+    Send a message to the specified in_route 
+    of this pattern's specifed in_block
+    """
+    
+    inblock = [b for b in self.blocks if b.id == self.in_block][0]
+    return self._st.to_block_route(in_block.id, route=self.in_route, msg=msg)
 
-  def stream(self):
-    outblock = [b for b in self.blocks if b.id == self.outblock][0]
-    return self._st.from_block_route(outblock.id, route=self.outroute, msg=msg)
+  def listen(self):
+
+    """
+    Listen for messages from the out route 
+    of this pattern's specifed out_block
+    """
+
+    out_block = [b for b in self.blocks if b.id == self.out_block][0]
+    return self._st.stream(out_block.id)
 
   def __add__(self, obj):
-    
+
+    """
+    Logic for adding Patterns + Connections.
+    """
+
     if isinstance(obj, Connection):
       
       # only add new conections + blocks to Pattern
@@ -405,6 +577,11 @@ class Pattern:
       return self
 
   def __repr__(self):
+
+    """
+    Pattern display
+    """
+
     return "< Pattern = Connections => {}, Blocks => {} >"\
       .format(self.connections, self.blocks)
 
